/* Color detection and palletizing for PhantomX Pincher Robot.
Authors: Andreas Kjær Jellesen, Heri Lamhauge Jensen, Esben Sigsgaard, Thai Huy Quang Nquyen
Students at University College Nordjylland class of Automation Engineering*/

//----------------------------------------Include-------------------------------------------//

#include "ax12.h"
#include "BioloidController.h"
#include "ServoAX.h"
#include <SPI.h>
#include <Pixy.h>

// Establishing connection speed to bioloid controller.
BioloidController bioloid = BioloidController(1000000);  

//----------------------------------------Variables-------------------------------------------//

//Global instance for the pixy camera
Pixy pixy;

//----------------------------------------Global Variables-------------------------------------------//
int steps = 10;
int color = 0;
int dropCount = 0;

//----------------------------------------Setup-------------------------------------------//

void setup()
{
   servoax.SETUP(); // calling setup function from servoax library 
   pixy.init(); // initializing pixy camera 
}

//----------------------------------------Main-------------------------------------------//

void loop ()
{
 if (digitalRead(0) == HIGH) // Activating pick up sequence
 {
   servoax.setPickUp(1);
 }
 
 if (digitalRead(2) == HIGH) // initialzes variable "steps" to '0', and by that starting the robot sequence
 {
   steps = 0;
 }
 
//----------------------------------------Sequence-------------------------------------------// 

 //Step 0 - Ready pos, open gripper
 if (steps == 0)
 {
    servoax.Playpose(0); 
    steps = 1;
  }

 //Step 1 - Pick up pos, closed gripper
 if (steps == 1)
 {
  servoax.Playpose(1);
  steps = 2;
 }
 
 //Step 2 - Ready pos, closed gripper
 if (steps == 2)
 {
  servoax.Playpose(0);
  steps = 3;
 }
 
 //Step 3 - Camera pos, closed gripper
 if (steps == 3)
 {
  servoax.Playpose(2);  
  color = servoax.getColor();
  steps = 4;
 }
 
 //Step 4 - Waypoint cam pos, closed gripper
 if (steps == 4)
 {
  servoax.Playpose(3);
  steps = 5;
 }
 
 //Step 5 - Ready pos, closed gripper
 if (steps == 5)
 {
   servoax.Playpose(0);
  
   //Choosing the drop position based on the return value/color from the function 
   steps = servoax.dropStep(color);
 }
 
 //Step 6 -  drop position for the first blue block, Open gripper
 if (steps == 6)
 { 
  servoax.Playpose(4);
  servoax.Playpose(8);
  color = 0;
  dropCount ++;  
  steps = 0;
 }
 
 //Step 7 -  drop position for the second blue block, Open gripper
 if (steps == 7)
 {
  servoax.Playpose(10);
  servoax.Playpose(5);
  servoax.Playpose(8);
  color = 0;
  dropCount ++;
  steps = 0;
 }
  
  //Step 8 -  drop position for the first green block, Open gripper
 if (steps == 8)
 {
  servoax.Playpose(9);
  servoax.Playpose(6);
  color = 0;
  dropCount ++;
  steps = 0;
 }
  
 //Step 9 -  drop position for the second green block, Open gripper
 if (steps == 9)
 {
  servoax.Playpose(11);  
  servoax.Playpose(7);
  servoax.Playpose(11);
  ax12SetRegister2(5 , 30, 512);
  color = 0;
  dropCount ++;
  steps = 0;
 }
 
 // this will stop the sequence when both pallets are full
 if (dropCount == 4)
 {
    servoax.Playpose(0);
    servoax.dropStep(3);
    Serial.println("###############################################");
    Serial.println("Pallet is full! Please empty it and press -RUN-");
    Serial.println("###############################################");
    Serial.print("");
    
    steps = 10;
    dropCount = 0;  
  }
}

//----------------------------------------End Main-------------------------------------------//
