/* Color detection and palletizing for PhantomX Pincher Robot.
Authors: Andreas Kjær Jellesen, Heri Lamhauge Jensen, Esben Sigsgaard, Thai Huy Quang Nquyen
Students at University College Nordjylland class of Automation Engineering*/

//----------------------------------------Include-------------------------------------------//
#include "arduino.h"
#include "servoax.h"
#include "ax12.h"
#include <SPI.h>  
#include <Pixy.h>

//----------------------------------------Functions-------------------------------------------//

servoclass::servoclass(){ // Including class
}

//----------------------------------------Variables-------------------------------------------//
int positions[12][5] = {{512, 299, 845, 717, 512}, // Pos 0 - Ready pos // 
                   {512, 512, 512, 0, 512},  // Pos 1 - Pick up pos - Close grip
                   {645, 650, 754, 674, 512}, // Pos 2 - Cam pos - Close grip
                   {645, 421, 767, 707, 511}, // Pos 3 - Waypoint cam pos - close grip
                   {210, 793, 574, 814, 512}, // Pos 4 - blue drop pos 1 - Release grip
                   {209, 645, 834, 661, 512}, // Pos 5 - blue drop pos 2 - Release grip
                   {514, 250, 430, 267, 512}, // Pos 6 - green drop pos 1 - Release grip
                   {514, 363, 207, 354, 512}, // Pos 7 - green drop pos 2 - Release grip
                   {204, 455, 820, 719, 512}, // Pos 8 - Waypoint while moving away from blue drop pos - Release Gripper
                   {514, 336, 411, 217, 512}, // Pos 9 - Waypoint to first green drop pos - Close grip
                   {215, 492, 820, 814, 512}, // Pos 10 - Waypoint to second blue drop - close grip
                   {514, 457, 242, 276, 512}}; // Pos 11 - Waypoint to  second green drop - close grip
int blueColor = 0;
int greenColor = 0;   
int velocity = 100;
                   
//----------------------------------------SETUP FUNCTION-------------------------------------------//
void servoclass::SETUP(){ // This is how to access functions made within the class from the .h file

   Serial.begin(9600); // opens serial port, sets data rate to 9600 bps
   delay (5000);   
   Serial.println("#################################");    
   Serial.println("Serial Communication Established.");
   Serial.println("#################################"); 
   Serial.println(" ");
   delay (500);
   
   ax12SetRegister2(254, 32, velocity);  // setting the velocity of servo
  
   //Digital inputs and output
   //pinMode(PIN, INPUT OR OUTPUT)
   pinMode(0, INPUT); //
   pinMode(1, INPUT);
   pinMode(2, INPUT);
   pinMode(3, OUTPUT);
   Serial.println("#####################");    
   Serial.println("Pin mode Established.");
   Serial.println("#####################"); 
   Serial.println(" ");
   delay (500);
   
   //Start setup of pick up point
   Serial.println("#################################################");
   Serial.println("Press the Setup button to set the Pick up postion");
   Serial.println("#################################################");
   Serial.println(" "); 
   
   // Turning on LED to Display for the user that the Robot is initialized
   digitalWrite(3, HIGH);
}

//----------------------------------------PLAYING A CHOOSEN POSE--------------------------------------//
int servoclass::Playpose(int pos)
{
  int moving;
  int griptool;
  
  ax12SetRegister2(254, 32, velocity);  // setting the velocity of servo
  delay(40);
  
   if (pos == 1) // The gripper is moving faster than the other servos when it is picking up the object
   {
     ax12SetRegister2(4, 32, velocity+40);  // setting the velocity of servo
   }
   
   for (int id = 0; id <= 3; id++) // Setting the position for all servos except the gripper which will be set when the robot is at the end distination.
   {
     delay(20);
     ax12SetRegister2(id + 1, 30, positions[pos][id]);
   } 
   
   moving=1;
   while (moving != 0) // The program will loop through this while loop until the robot is at its end position and not moving
   {
     delay(100);
     moving = ax12GetRegister(1,46,1)+ax12GetRegister(2,46,1)+ax12GetRegister(3,46,1)+ax12GetRegister(4,46,1);// checking if the motors are running. If the result is == 0 no servos is moving
   }
   
   griptool = 1;
   if (pos != 0)
   {
     while (griptool != 0) // // The program will loop through this while loop until the gripper is at its end position
     { 
       ax12SetRegister2(5 , 30, positions[pos][4]); // Playing gripper pose
       delay(300);
       griptool = ax12GetRegister(5,46,1);
     }
   }  
}

//----------------------------------------SETTING THE PICK UP POSITION-------------------------------------------//

int servoclass::setPickUp(int start)
{
  for (int id =1; id <=4; id++) // Relaxing all servos except the gripper
  {
    ax12SetRegister2(id, 24, 0);
    delay(25);
  } 
  
  Serial.println("###########################################################################");    
  Serial.println("All servos are relaxed. Please move the robot to the pick up position! :-) ");
  Serial.println("Press -SET POS- Button to save the pick up pose");
  
  while(positions[1][3] < 1) // This array place is initialized with '0' because nothing is stored in there before the user has set the pick up position. When the position is set the loop will terminate.
  {
    delay(10);
    ax12SetRegister2(5, 30, map(analogRead(0), 0, 1023, 0, 512));  // setting the gripper postion
  
  if (digitalRead(1) == HIGH) // This input will set the pick up position
  {
    for (int id =0; id <=3; id++) 
    {
      ax12SetRegister2(id, 24, 1);
      delay(20);
      positions[1][id] = ax12GetRegister(id+1, 36, 2);
    }
  // Saving positions for gripper! - the program will automaticly save the gripper position in the steps where it is holding the object. In any other step the gripper will just remain open. 
  positions[1][4] = ax12GetRegister(5, 36, 2); 
  positions[2][4] = ax12GetRegister(5, 36, 2);
  positions[3][4] = ax12GetRegister(5, 36, 2);
  positions[9][4] = ax12GetRegister(5, 36, 2);
  positions[10][4] = ax12GetRegister(5, 36, 2);
  positions[11][4] = ax12GetRegister(5, 36, 2);
  
  }
  }
     ax12SetRegister2(5, 30, 512); // releases the gripper after setting the pick up position. this will allow the user to remove the robot without the object stuck in the gripper.
     Serial.println("The pick up pose is saved!");
     Serial.println("###########################################################################"); 
     Serial.println("");
     Serial.println("############################"); 
     Serial.println("Press -RUN- to start program");
     Serial.println("############################");
     Serial.println("");
}

//----------------------------------------Choice drop of step-------------------------------------------//

int servoclass::dropStep(int color)
{
  // this will clear previously saved droppositions so the sequence can run again
  if (color == 3)
  {
    blueColor = 0;
    greenColor = 0;
  }
  //Blue blocks
  if (color == 1)
  {
    //First blue block
    if (blueColor == 0)
    {
      blueColor++;
      return 6; // return next step value
    }
    
    //Second blue block
    else if (blueColor == 1)
    {
      blueColor++; 
      return 7; // returns next step value
    }
  }
  
  //Green blocks
  else if (color == 2)
  {
    //First green block
    if (greenColor == 0)
    {
      greenColor++;
      return 8; //returns next step value
    }
    
    
    //Second green block
    else if (greenColor == 1)
    {
      greenColor++;
      return 9; // returns next step value
    }
  }
}
//----------------------------------------Get color-------------------------------------------//  

int servoclass::getColor ()
{
  Pixy pixy;
  pixy.init(); // initializing pixy camera
  delay(20);
  
  int i = 0;
  int blocks;
  int color;
  int j;
 
 while (color != 1 || color != 2) // as long as there is no color detected from the pixy camera this loop will continue.
 { 
  // grab blocks!
  blocks = pixy.getBlocks();
  
  // If there are detect blocks the if statement will execute
  if (blocks)
  {
    i++;
    
    // Check for objects every 20 frames to avoid overloading the arduino 
    if (i%20==0)
    {
       color = pixy.blocks[j].signature;  
       return color;  
    }
  }
 }
}  
  servoclass servoax = servoclass(); // Also here the compiler will need this to reference to the name servoax 

